#pragma once

// A tiny little stub math library using just regular floating point,
// just so we have a baseline to compare against.

struct vec3 { float x, y, z; };

// min/max are annoying as they usually involve branches. 
// Even using just regular FP math, you can optimize this by
// using SSE helpers as shown below. _mm_set_ss and _mm_cvtss_f32
// aren't actual instructions, they're just casts, so this whole function
// just optimizes down to a single instruction. You can pick whichever
// you like. I've left it at the simple implementation as in this example
// it doesn't help as much as you might think.
#if 1
VM_INLINE float min(float a, float b) { return (a < b) ? a : b; }
VM_INLINE float max(float a, float b) { return (a > b) ? a : b; }
#else
VM_INLINE float min(float a, float b) {
	__m128 m = _mm_min_ss(_mm_set_ss(a), _mm_set_ss(b));
	return _mm_cvtss_f32(m);
}
VM_INLINE float max(float a, float b) {
	__m128 m = _mm_max_ss(_mm_set_ss(a), _mm_set_ss(b));
	return _mm_cvtss_f32(m);
}
#endif

VM_INLINE vec3 operator- (vec3 a, vec3 b) { vec3 t = { a.x-b.x, a.y-b.y, a.z-b.z }; return t; }
VM_INLINE vec3 operator* (vec3 a, vec3 b) { vec3 t = { a.x*b.x, a.y*b.y, a.z*b.z }; return t; }
VM_INLINE vec3 operator/ (vec3 a, vec3 b) { vec3 t = { a.x/b.x, a.y/b.y, a.z/b.z }; return t; }
VM_INLINE vec3 min(vec3 a, vec3 b) { vec3 t = { min(a.x, b.x), min(a.y, b.y), min(a.z, b.z) }; return t; }
VM_INLINE vec3 max(vec3 a, vec3 b) { vec3 t = { max(a.x, b.x), max(a.y, b.y), max(a.z, b.z) }; return t; }
VM_INLINE float hmin(vec3 a) { return min(min(a.x, a.y), a.z); }
VM_INLINE float hmax(vec3 a) { return max(max(a.x, a.y), a.z); }

